from ftw.calendar.browser.interfaces import IFtwCalendarLayer
from ftw.calendar.testing import FTW_CALENDAR_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from unittest2 import TestCase
from zope.interface import directlyProvides


class TestCalendarView(TestCase):

    layer = FTW_CALENDAR_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

    def test_calendarview_available(self):
        directlyProvides(self.portal.REQUEST, IFtwCalendarLayer)
        view = self.portal.restrictedTraverse('@@ftwcalendar_view')

        self.assertTrue(view, 'ftw calendar view is not available')

    @browsing
    def test_calendar_view(self, browser):
        browser.visit(self.portal, view='ftwcalendar_view')

        element = browser.css('#calendar')
        self.assertTrue(element, 'Not div with if "calendar" found')



from ftw.testing.genericsetup import GenericSetupUninstallMixin
from ftw.testing.genericsetup import apply_generic_setup_layer
from unittest2 import TestCase


@apply_generic_setup_layer
class TestGenericSetupUninstall(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.calendar'
    install_dependencies = False

    def _install_package(self):
        self.setup_tool.runAllImportStepsFromProfile(
            'profile-ftw.calendar:general', ignore_dependencies=True)
        self.setup_tool.runAllImportStepsFromProfile(
            'profile-ftw.calendar:default', ignore_dependencies=True)



from ftw.builder import builder_registry
from ftw.builder.archetypes import ArchetypesBuilder
from ftw.builder.dexterity import DexterityBuilder
from Products.CMFPlone.utils import getFSVersionTuple


class EventBuilderMixin(object):
    portal_type = 'Event'


class ATEventBuilder(EventBuilderMixin, ArchetypesBuilder):
    pass


class DXEventBuilder(EventBuilderMixin, DexterityBuilder):
    pass


if getFSVersionTuple() > (5, ):
    builder_registry.register('event', DXEventBuilder)
else:
    builder_registry.register('event', ATEventBuilder)



from zope.interface import Interface


class IFtwCalendarLayer(Interface):
    """Marker interface that defines a Zope 3 browser layer.
    """


class IFtwCalendarJSONSourceProvider(Interface):
    """ Provides the JSON source to fill the calendar with events.
    """

    def generate_json_calendar_source():
        """ Calls the following methods to get the calendar source and returns
            is as JSON.
        """

    def get_event_brains():
        """ Returns list of brains to display in the current calendar view.
        """

    def generate_source_dict_from_brain(brain):
        """ Reads the relevant information from the brain and generates a dict
            based on thos informations. This dict is later converted to JSON.
        """


class IFtwCalendarEventCreator(Interface):

    def getEventType():
        """ Get the calendar event type name to be adden when
            user clicks in calendar.

        @return: event type as string
        """

    def createEvent(title, start_date):
        """ Creates a calendar event in the given context and data.

        @return: the newly created event
        """



from ftw.builder import Builder
from ftw.builder import create
from ftw.calendar.browser.calendarupdateview import CalendarJSONSource
from ftw.calendar.browser.interfaces import IFtwCalendarJSONSourceProvider
from ftw.calendar.testing import FTW_CALENDAR_FUNCTIONAL_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from unittest2 import TestCase
from zope.component import getMultiAdapter
from zope.interface.verify import verifyObject


class TestCalendarSource(TestCase):

    layer = FTW_CALENDAR_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.event = create(Builder('event'))

    def test_adapter_provides_methods(self):
        verifyObject(IFtwCalendarJSONSourceProvider, CalendarJSONSource(
            self.portal, self.portal.REQUEST))

    def test_default_json_source_adapter(self):
        adapter = getMultiAdapter((self.portal, self.portal.REQUEST),
                                  name='ftw_calendar_source')

        self.assertEquals(1,
                          len(adapter.get_event_brains()))

        brain = adapter.get_event_brains()[0]

        self.assertEquals(
            ['className', 'start', 'allDay', 'end', 'description',
             'title', 'url', 'editable', 'id'],
            adapter.generate_source_dict_from_brain(brain).keys())



from DateTime import DateTime
from datetime import timedelta
from ftw.calendar.browser.interfaces import IFtwCalendarEventCreator
from ftw.calendar.browser.interfaces import IFtwCalendarJSONSourceProvider
from plone import api
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.interface import implements
import simplejson as json


class CalendarJSONSource(object):
    implements(IFtwCalendarJSONSourceProvider)

    def __init__(self, context, request):
        self.context = context
        self.request = request

        self.memberid = \
            self.context.portal_membership.getAuthenticatedMember().id

    def generate_json_calendar_source(self):
        result = []

        for brain in self.get_event_brains():
            result.append(self.generate_source_dict_from_brain(brain))

        return json.dumps(result, sort_keys=True)

    def get_event_brains(self):
        args = {
            'start': {
                'query': DateTime(self.request.get('end')), 'range': 'max'},
            'end': {
                'query': DateTime(self.request.get('start')), 'range': 'min'}}
        if self.context.portal_type in ['Topic', 'Collection']:
            return self.context.aq_inner.queryCatalog(args)
        else:
            catalog = getToolByName(self.context, 'portal_catalog')
            portal_calendar = getToolByName(self.context, 'portal_calendar',
                                            None)
            if portal_calendar:
                args['portal_type'] = portal_calendar.getCalendarTypes()

            return catalog(
                path={'depth': -1,
                      'query': '/'.join(self.context.getPhysicalPath())},
                **args
            )

    def generate_source_dict_from_brain(self, brain):
        #  plone 4-5 compat
        creator = brain.Creator
        if callable(creator):
            creator = creator()
        title = brain.Title
        if callable(title):
            title = title()
        description = brain.Description
        if callable(description):
            description = description()
        start = brain.start
        end = brain.end
        iso = hasattr(start, 'isoformat') and 'isoformat' or 'ISO8601'
        start = getattr(start, iso)()
        end = getattr(end, iso)()

        event = brain.getObject()
        editable = api.user.has_permission('Edit', obj=event)

        # The default source marks an event as all day if it is longer than
        # one day. Marking an event as all day in ftw.contentpage will set
        # the times to 00:00 and 23:59. If those times are on the same
        # date they will not be recognised as all day because thats only a
        # 0.999.. day. This check will mark those events as all day.
        duration = brain.end - brain.start
        if isinstance(duration, timedelta):
            duration = duration.seconds / 60. / 60.
        allday = duration > 0.99

        return {"id": "UID_%s" % (brain.UID),
                "title": title,
                "start": start,
                "end": end,
                "url": brain.getURL(),
                "editable": editable,
                "allDay": allday,
                "className": "state-" + str(brain.review_state)
                + (editable and " editable" or ""),
                "description": description}


class CalendarupdateView(BrowserView):
    """ Calendarupdate browser view
    """

    def __call__(self, *args, **kw):
        """Render JS Initialization code"""

        response = self.request.response

        source_provider = getMultiAdapter((self.context, self.request),
                                          name='ftw_calendar_source')

        response.setHeader('Content-Type', 'application/json')
        return source_provider.generate_json_calendar_source()


class CalendarDropView(BrowserView):
    def __call__(self):
        request = self.context.REQUEST

        event_uid = request.get('event')

        if event_uid:
            event_uid = event_uid.split('UID_')[1]
        brains = self.context.portal_catalog(UID=event_uid)

        obj = brains[0].getObject()
        startDate, endDate = obj.startDate, obj.endDate
        dayDelta, minuteDelta = float(request.get('dayDelta')), \
                                float(request.get('minuteDelta'))

        startDate = startDate + dayDelta + minuteDelta / 1440.0
        endDate = endDate + dayDelta + minuteDelta / 1440.0

        obj.setStartDate(startDate)
        obj.setEndDate(endDate)
        obj.reindexObject()
        return True


class CalendarResizeView(BrowserView):
    def __call__(self):
        request = self.context.REQUEST
        event_uid = request.get('event')
        if event_uid:
            event_uid = event_uid.split('UID_')[1]
        brains = self.context.portal_catalog(UID=event_uid)
        obj = brains[0].getObject()
        endDate = obj.endDate
        dayDelta, minuteDelta = float(request.get('dayDelta')), \
                                float(request.get('minuteDelta'))

        endDate = endDate + dayDelta + minuteDelta / 1440.0

        obj.setEndDate(endDate)
        obj.reindexObject()
        return True


class CalendarAddView(BrowserView):
    def __call__(self):
        request = self.context.REQUEST
        title = request.get('title')
        start_date = DateTime(int(request.get('startdate')))

        if not title or not start_date:
            raise ValueError()

        eventCreator = getMultiAdapter((self.context, request),
                                       IFtwCalendarEventCreator)
        event = eventCreator.createEvent(title, start_date)

        self.request.RESPONSE.redirect(event.absolute_url() + '/edit')



from ftw.upgrade import UpgradeStep


class FixFullcalendarJsMinification(UpgradeStep):
    """Fix fullcalendar js minification
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.calendar.browser.interfaces import IFtwCalendarEventCreator
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from zope.component import ComponentLookupError
from zope.component import getMultiAdapter
from zope.component.hooks import getSite


class CalendarConfigView(BrowserView):
    """
    Calendar configuration view.
    """

    def __call__(self):
        self.request.response.setHeader('X-Theme-Disabled', 'True')
        return super(CalendarConfigView, self).__call__()

    def first_day(self):
        """
        Returns the first day of the week as an integer.
        """
        calendar_tool = getToolByName(self.context, 'portal_calendar', None)
        if calendar_tool:
            first = calendar_tool.getFirstWeekDay()
        else:
            first = getSite().portal_registry['plone.first_weekday']
        return (first < 6 and first + 1) or 0

    def can_add_content(self):
        try:
            eventCreator = getMultiAdapter((self.context, self.request),
                                           IFtwCalendarEventCreator)
        except ComponentLookupError:
            return False

        return eventCreator.getEventType() in \
               self.context.getImmediatelyAddableTypes()



# -*- coding: utf-8 -*-
from Products.CMFPlone import interfaces as Plone
from Products.CMFQuickInstallerTool import interfaces as QuickInstaller
from zope.interface import implementer


@implementer(Plone.INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Do not show on Plone's list of installable profiles."""
        return [
            u'ftw.calendar:base',
            u'ftw.calendar:uninstall',
            u'ftw.calendar:uninstall-base',
        ]


@implementer(QuickInstaller.INonInstallable)
class HiddenProducts(object):

    def getNonInstallableProducts(self):
        """Do not show on QuickInstaller's list of installable products."""
        return [
        ]



from setuptools import setup, find_packages
import os

version = '3.1.2'

tests_require = ['ftw.testing',
                 'ftw.builder',
                 'plone.app.testing',
                 'ftw.testbrowser',
                 ]


setup(name='ftw.calendar',
      version=version,
      description="Calendar view based on fullcalendar",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        'Framework :: Plone :: 4.3',
        'Framework :: Plone :: 5.1',
        "Programming Language :: Python",
        ],
      keywords='',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.calendar',
      license='GPL2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      install_requires=[
          'Plone',
          'setuptools',
          'ftw.upgrade',
          'simplejson',
          'plone.api',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )
